@@ -46,6 +46,7 @@ typedef struct _gki_globals {
 	gchar **full_group_names;
 	gchar **short_group_names;
 	GSList *widget_instances;
+	GSList *images;
 } gki_globals;
 
 struct _GkbdIndicatorPrivate {
@@ -84,19 +85,104 @@ gkbd_indicator_fill (GkbdIndicator * gki);
 static void
 gkbd_indicator_set_tooltips (GkbdIndicator * gki, const char *str);
 
+void
+gkbd_indicator_load_images ()
+{
+	int i;
+	GSList *image_filename;
+
+	globals.images = NULL;
+	gkbd_indicator_config_load_image_filenames (&globals.ind_cfg,
+						    &globals.kbd_cfg);
+
+	if (!globals.ind_cfg.show_flags)
+		return;
+
+	image_filename = globals.ind_cfg.image_filenames;
+
+	for (i = xkl_engine_get_max_num_groups (globals.engine);
+	     --i >= 0; image_filename = image_filename->next) {
+		GdkPixbuf *image = NULL;
+		char *image_file = (char *) image_filename->data;
+
+		if (image_file != NULL) {
+			GError *gerror = NULL;
+			image =
+			    gdk_pixbuf_new_from_file (image_file, &gerror);
+			if (image == NULL) {
+				GtkWidget *dialog =
+				    gtk_message_dialog_new (NULL,
+							    GTK_DIALOG_DESTROY_WITH_PARENT,
+							    GTK_MESSAGE_ERROR,
+							    GTK_BUTTONS_OK,
+							    _
+							    ("There was an error loading an image: %s"),
+							    gerror->
+							    message);
+				g_signal_connect (G_OBJECT (dialog),
+						  "response",
+						  G_CALLBACK
+						  (gtk_widget_destroy),
+						  NULL);
+
+				gtk_window_set_resizable (GTK_WINDOW
+							  (dialog), FALSE);
+
+				gtk_widget_show (dialog);
+				g_error_free (gerror);
+			}
+			xkl_debug (150,
+				   "Image %d[%s] loaded -> %p[%dx%d]\n",
+				   i, image_file, image,
+				   gdk_pixbuf_get_width (image),
+				   gdk_pixbuf_get_height (image));
+		}
+		/* We append the image anyway - even if it is NULL! */
+		globals.images = g_slist_append (globals.images, image);
+	}
+}
+
+static void
+gkbd_indicator_free_images ()
+{
+	GdkPixbuf *pi;
+	GSList *img_node;
+
+	gkbd_indicator_config_free_image_filenames (&globals.ind_cfg);
+
+	while ((img_node = globals.images) != NULL) {
+		pi = GDK_PIXBUF (img_node->data);
+		/* It can be NULL - some images may be missing */
+		if (pi != NULL) {
+			g_object_unref (pi);
+		}
+		globals.images =
+		    g_slist_remove_link (globals.images, img_node);
+		g_slist_free_1 (img_node);
+	}
+}
+
+static void
+gkbd_indicator_update_images (void)
+{
+	gkbd_indicator_free_images ();
+	gkbd_indicator_load_images ();
+}
+
 void
 gkbd_indicator_set_tooltips (GkbdIndicator * gki, const char *str)
 {
-        g_assert (str == NULL || g_utf8_validate (str, -1, NULL));
+	g_assert (str == NULL || g_utf8_validate (str, -1, NULL));
 
-        gtk_widget_set_tooltip_text (GTK_WIDGET (gki), str);
+	gtk_widget_set_tooltip_text (GTK_WIDGET (gki), str);
 
 	if (gki->priv->set_parent_tooltips) {
-		GtkWidget *parent = gtk_widget_get_parent (GTK_WIDGET (gki));
-                if (parent) {
-                        gtk_widget_set_tooltip_text (parent, str);
-                }
-        }
+		GtkWidget *parent =
+		    gtk_widget_get_parent (GTK_WIDGET (gki));
+		if (parent) {
+			gtk_widget_set_tooltip_text (parent, str);
+		}
+	}
 }
 
 void
@@ -122,8 +208,7 @@ gkbd_indicator_fill (GkbdIndicator * gki)
 		GtkWidget *page, *decorated_page = NULL;
 		gchar *full_group_name =
 		    (grp <
-		     g_strv_length (globals.
-				    full_group_names)) ?
+		     g_strv_length (globals.full_group_names)) ?
 		    globals.full_group_names[grp] : "?";
 		page = gkbd_indicator_prepare_drawing (gki, grp);
 
@@ -168,8 +253,10 @@ gkbd_indicator_button_pressed (GtkWidget *
 			       GdkEventButton * event, GkbdIndicator * gki)
 {
 	GtkWidget *img = gtk_bin_get_child (GTK_BIN (widget));
+	GtkAllocation allocation;
+	gtk_widget_get_allocation (img, &allocation);
 	xkl_debug (150, "Flag img size %d x %d\n",
-		   img->allocation.width, img->allocation.height);
+		   allocation.width, allocation.height);
 	if (event->button == 1 && event->type == GDK_BUTTON_PRESS) {
 		xkl_debug (150, "Mouse button pressed on applet\n");
 		gkbd_desktop_config_lock_next_group (&globals.cfg);
@@ -184,9 +271,18 @@ flag_exposed (GtkWidget * flag, GdkEventExpose * event, GdkPixbuf * image)
 	/* Image width and height */
 	int iw = gdk_pixbuf_get_width (image);
 	int ih = gdk_pixbuf_get_height (image);
+	GtkAllocation allocation;
+	gboolean scaling_needed;
+
+	gtk_widget_get_allocation (flag, &allocation);
+	scaling_needed =
+	    !((allocation.width == iw &&
+	       allocation.height <= ih) ||
+	      (allocation.width >= iw && allocation.height == ih));
+
 	/* widget-to-image scales, X and Y */
-	double xwiratio = 1.0 * flag->allocation.width / iw;
-	double ywiratio = 1.0 * flag->allocation.height / ih;
+	double xwiratio = 1.0 * allocation.width / iw;
+	double ywiratio = 1.0 * allocation.height / ih;
 	double wiratio = xwiratio < ywiratio ? xwiratio : ywiratio;
 
 	/* scaled width and height */
@@ -194,18 +290,23 @@ flag_exposed (GtkWidget * flag, GdkEventExpose * event, GdkPixbuf * image)
 	int sh = ih * wiratio;
 
 	/* offsets */
-	int ox = (flag->allocation.width - sw) >> 1;
-	int oy = (flag->allocation.height - sh) >> 1;
+	int ox = (allocation.width - sw) >> 1;
+	int oy = (allocation.height - sh) >> 1;
 
-	GdkPixbuf *scaled = gdk_pixbuf_scale_simple (image, sw, sh,
-						     GDK_INTERP_HYPER);
+	GdkPixbuf *scaled =
+	    scaling_needed ? gdk_pixbuf_scale_simple (image, sw, sh,
+						      GDK_INTERP_HYPER) :
+	    image;
 
-	gdk_draw_pixbuf (GDK_DRAWABLE (flag->window),
+	gdk_draw_pixbuf (GDK_DRAWABLE (gtk_widget_get_window (flag)),
 			 NULL,
 			 scaled,
 			 0, 0,
-			 ox, oy, sw, sh, GDK_RGB_DITHER_NORMAL, 0, 0);
-	g_object_unref (G_OBJECT (scaled));
+			 ox, oy, sw, sh,
+			 scaling_needed ? GDK_RGB_DITHER_NORMAL :
+			 GDK_RGB_DITHER_NONE, 0, 0);
+	if (scaling_needed)
+		g_object_unref (G_OBJECT (scaled));
 }
 
 static GtkWidget *
@@ -215,16 +316,18 @@ gkbd_indicator_prepare_drawing (GkbdIndicator * gki, int group)
 	GdkPixbuf *image;
 	GtkWidget *ebox;
 
-	pimage = g_slist_nth_data (globals.ind_cfg.images, group);
+	pimage = g_slist_nth_data (globals.images, group);
 	ebox = gtk_event_box_new ();
+	gtk_event_box_set_visible_window (GTK_EVENT_BOX (ebox), FALSE);
 	if (globals.ind_cfg.show_flags) {
 		GtkWidget *flag;
 		if (pimage == NULL)
 			return NULL;
 		image = GDK_PIXBUF (pimage);
 		flag = gtk_drawing_area_new ();
-		g_signal_connect (G_OBJECT (flag),
-				  "expose_event",
+		gtk_widget_add_events (GTK_WIDGET (flag),
+				       GDK_BUTTON_PRESS_MASK);
+		g_signal_connect (G_OBJECT (flag), "expose_event",
 				  G_CALLBACK (flag_exposed), image);
 		gtk_container_add (GTK_CONTAINER (ebox), flag);
 	} else {
@@ -325,6 +428,7 @@ gkbd_indicator_prepare_drawing (GkbdIndicator * gki, int group)
 
 		gtk_container_set_border_width (GTK_CONTAINER (align), 2);
 	}
+
 	g_signal_connect (G_OBJECT (ebox),
 			  "button_press_event",
 			  G_CALLBACK (gkbd_indicator_button_pressed), gki);
@@ -394,8 +498,7 @@ gkbd_indicator_ind_cfg_changed (GConfClient * client,
 	xkl_debug (100,
 		   "Applet configuration changed in GConf - reiniting...\n");
 	gkbd_indicator_config_load_from_gconf (&globals.ind_cfg);
-	gkbd_indicator_config_update_images (&globals.ind_cfg,
-					     &globals.kbd_cfg);
+	gkbd_indicator_update_images ();
 	gkbd_indicator_config_activate (&globals.ind_cfg);
 
 	gkbd_indicator_plugin_manager_toggle_plugins
@@ -447,8 +550,7 @@ gkbd_indicator_kbd_cfg_callback (GkbdIndicator * gki)
 
 	gkbd_keyboard_config_load_from_x_current (&globals.kbd_cfg,
 						  xklrec);
-	gkbd_indicator_config_update_images (&globals.ind_cfg,
-					     &globals.kbd_cfg);
+	gkbd_indicator_update_images ();
 
 	g_strfreev (globals.full_group_names);
 	globals.full_group_names = NULL;
@@ -563,7 +665,7 @@ gkbd_indicator_start_listen (void)
 static void
 gkbd_indicator_stop_listen (void)
 {
-	xkl_engine_stop_listen (globals.engine);
+	xkl_engine_stop_listen (globals.engine, XKLL_TRACK_KEYBOARD_STATE);
 
 	gdk_window_remove_filter (NULL, (GdkFilterFunc)
 				  gkbd_indicator_filter_x_evt, NULL);
@@ -740,14 +842,14 @@ gkbd_indicator_global_init (void)
 
 	globals.registry =
 	    xkl_config_registry_get_instance (globals.engine);
-	xkl_config_registry_load (globals.registry, globals.cfg.load_extra_items);
+	xkl_config_registry_load (globals.registry,
+				  globals.cfg.load_extra_items);
 
 	gkbd_keyboard_config_load_from_x_current (&globals.kbd_cfg,
 						  xklrec);
 
 	gkbd_indicator_config_load_from_gconf (&globals.ind_cfg);
-	gkbd_indicator_config_update_images (&globals.ind_cfg,
-					     &globals.kbd_cfg);
+	gkbd_indicator_update_images ();
 	gkbd_indicator_config_activate (&globals.ind_cfg);
 
 	gkbd_indicator_load_group_names ((const gchar **) xklrec->layouts,
@@ -821,7 +923,7 @@ gdouble
 gkbd_indicator_get_max_width_height_ratio (void)
 {
 	gdouble rv = 0.0;
-	GSList *ip = globals.ind_cfg.images;
+	GSList *ip = globals.images;
 	if (!globals.ind_cfg.show_flags)
 		return 0;
 	while (ip != NULL) {
