@@ -998,8 +998,8 @@ draw_key (GkbdKeyboardDrawingRenderContext * context,
 
 	if (key->pressed)
 		color =
-		    &(GTK_WIDGET (drawing)->style->
-		      base[GTK_STATE_SELECTED]);
+		    &gtk_widget_get_style (GTK_WIDGET (drawing))->base
+		    [GTK_STATE_SELECTED];
 	else
 		color = drawing->colors + key->xkbkey->color_ndx;
 
@@ -1097,9 +1097,8 @@ invalidate_indicator_doodad_region (GkbdKeyboardDrawing * drawing,
 			   doodad->doodad->indicator.left,
 			   doodad->origin_y +
 			   doodad->doodad->indicator.top,
-			   &drawing->xkb->geom->shapes[doodad->doodad->
-						       indicator.
-						       shape_ndx]);
+			   &drawing->xkb->geom->shapes[doodad->
+						       doodad->indicator.shape_ndx]);
 }
 
 static void
@@ -1113,8 +1112,8 @@ invalidate_key_region (GkbdKeyboardDrawing * drawing,
 			   key->angle,
 			   key->origin_x,
 			   key->origin_y,
-			   &drawing->xkb->geom->shapes[key->xkbkey->
-						       shape_ndx]);
+			   &drawing->xkb->geom->shapes[key->
+						       xkbkey->shape_ndx]);
 }
 
 static void
@@ -1266,7 +1265,11 @@ draw_keyboard_item (GkbdKeyboardDrawingItem * item,
 		return;
 
 	switch (item->type) {
+	case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_INVALID:
+		break;
+
 	case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY:
+	case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY_EXTRA:
 		draw_key (context, drawing,
 			  (GkbdKeyboardDrawingKey *) item);
 		break;
@@ -1293,11 +1296,11 @@ draw_keyboard_to_context (GkbdKeyboardDrawingRenderContext * context,
 static void
 create_cairo (GkbdKeyboardDrawing * drawing)
 {
-	GtkStateType state = GTK_WIDGET_STATE (GTK_WIDGET (drawing));
+	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
 	drawing->renderContext->cr =
 	    gdk_cairo_create (GDK_DRAWABLE (drawing->pixmap));
 	drawing->renderContext->dark_color =
-	    &GTK_WIDGET (drawing)->style->dark[state];
+	    &gtk_widget_get_style (GTK_WIDGET (drawing))->dark[state];
 }
 
 static void
@@ -1311,22 +1314,24 @@ destroy_cairo (GkbdKeyboardDrawing * drawing)
 static void
 draw_keyboard (GkbdKeyboardDrawing * drawing)
 {
-	GtkStateType state = GTK_WIDGET_STATE (GTK_WIDGET (drawing));
-	gint pixw, pixh;
+	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
+	GtkAllocation allocation;
 
 	if (!drawing->xkb)
 		return;
 
-	pixw = GTK_WIDGET (drawing)->allocation.width;
-	pixh = GTK_WIDGET (drawing)->allocation.height;
+	gtk_widget_get_allocation (GTK_WIDGET (drawing), &allocation);
 
 	drawing->pixmap =
-	    gdk_pixmap_new (GTK_WIDGET (drawing)->window, pixw, pixh, -1);
+	    gdk_pixmap_new (gtk_widget_get_window (GTK_WIDGET (drawing)),
+			    allocation.width, allocation.height, -1);
 
 	/* blank background */
 	gdk_draw_rectangle (drawing->pixmap,
-			    GTK_WIDGET (drawing)->style->base_gc[state],
-			    TRUE, 0, 0, pixw, pixh);
+			    gtk_widget_get_style (GTK_WIDGET
+						  (drawing))->base_gc
+			    [state], TRUE, 0, 0, allocation.width,
+			    allocation.height);
 
 	create_cairo (drawing);
 	draw_keyboard_to_context (drawing->renderContext, drawing);
@@ -1346,8 +1351,9 @@ alloc_render_context (GkbdKeyboardDrawing * drawing)
 	pango_layout_set_ellipsize (context->layout, PANGO_ELLIPSIZE_END);
 
 	context->font_desc =
-	    pango_font_description_copy (GTK_WIDGET (drawing)->style->
-					 font_desc);
+	    pango_font_description_copy (gtk_widget_get_style
+					 (GTK_WIDGET
+					  (drawing))->font_desc);
 	context->angle = 0;
 	context->scale_numerator = 1;
 	context->scale_denominator = 1;
@@ -1368,7 +1374,8 @@ static gboolean
 expose_event (GtkWidget * widget,
 	      GdkEventExpose * event, GkbdKeyboardDrawing * drawing)
 {
-	GtkStateType state = GTK_WIDGET_STATE (GTK_WIDGET (drawing));
+	GtkStateType state = gtk_widget_get_state (GTK_WIDGET (drawing));
+	GtkAllocation allocation;
 
 	if (!drawing->xkb)
 		return FALSE;
@@ -1376,20 +1383,22 @@ expose_event (GtkWidget * widget,
 	if (drawing->pixmap == NULL)
 		return FALSE;
 
-	gdk_draw_drawable (widget->window,
-			   widget->style->fg_gc[state],
+	gdk_draw_drawable (gtk_widget_get_window (widget),
+			   gtk_widget_get_style (widget)->fg_gc[state],
 			   drawing->pixmap,
 			   event->area.x, event->area.y,
 			   event->area.x, event->area.y,
 			   event->area.width, event->area.height);
 
-	if (GTK_WIDGET_HAS_FOCUS (widget))
-		gtk_paint_focus (widget->style, widget->window,
-				 GTK_WIDGET_STATE (widget), &event->area,
-				 widget, "keyboard-drawing",
-				 0, 0,
-				 widget->allocation.width,
-				 widget->allocation.height);
+	if (gtk_widget_has_focus (widget)) {
+		gtk_widget_get_allocation (widget, &allocation);
+		gtk_paint_focus (gtk_widget_get_style (widget),
+				 gtk_widget_get_window (widget),
+				 gtk_widget_get_state (widget),
+				 &event->area, widget, "keyboard-drawing",
+				 0, 0, allocation.width,
+				 allocation.height);
+	}
 
 	return FALSE;
 }
@@ -1495,8 +1504,6 @@ key_event (GtkWidget * widget,
 	destroy_cairo (drawing);
 
 	invalidate_key_region (drawing, key);
-
-	printf("fu\n");
 	return FALSE;
 }
 
@@ -1664,8 +1671,8 @@ init_keys_and_doodads (GkbdKeyboardDrawing * drawing)
 				    drawing->xkb->geom->shapes +
 				    xkbkey->shape_ndx;
 				guint keycode = find_keycode (drawing,
-							      xkbkey->name.
-							      name);
+							      xkbkey->
+							      name.name);
 
 				if (keycode == INVALID_KEYCODE)
 					continue;
@@ -1682,9 +1689,24 @@ init_keys_and_doodads (GkbdKeyboardDrawing * drawing)
 
 				if (keycode >= drawing->xkb->min_key_code
 				    && keycode <=
-				    drawing->xkb->max_key_code)
+				    drawing->xkb->max_key_code) {
 					key = drawing->keys + keycode;
-				else {
+					if (key->type ==
+					    GKBD_KEYBOARD_DRAWING_ITEM_TYPE_INVALID)
+					{
+						key->type =
+						    GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY;
+					} else {
+						/* duplicate key for the same keycode, 
+						   already defined as GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY */
+						key =
+						    g_new0
+						    (GkbdKeyboardDrawingKey,
+						     1);
+						key->type =
+						    GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY_EXTRA;
+					}
+				} else {
 					g_warning
 					    ("key %4.4s: keycode = %u; not in range %d..%d\n",
 					     xkbkey->name.name, keycode,
@@ -1694,10 +1716,10 @@ init_keys_and_doodads (GkbdKeyboardDrawing * drawing)
 					key =
 					    g_new0 (GkbdKeyboardDrawingKey,
 						    1);
+					key->type =
+					    GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY_EXTRA;
 				}
 
-				key->type =
-				    GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY;
 				key->xkbkey = xkbkey;
 				key->angle = section->angle;
 				rotate_coordinate (section->left,
@@ -1761,8 +1783,9 @@ init_colors (GkbdKeyboardDrawing * drawing)
 
 	for (i = 0; i < drawing->xkb->geom->num_colors; i++) {
 		result =
-		    parse_xkb_color_spec (drawing->xkb->geom->colors[i].
-					  spec, drawing->colors + i);
+		    parse_xkb_color_spec (drawing->xkb->geom->
+					  colors[i].spec,
+					  drawing->colors + i);
 
 		if (!result)
 			g_warning
@@ -1782,19 +1805,15 @@ free_cdik (			/*colors doodads indicators keys */
 
 	for (itemp = drawing->keyboard_items; itemp; itemp = itemp->next) {
 		GkbdKeyboardDrawingItem *item = itemp->data;
-		GkbdKeyboardDrawingKey *key;
 
 		switch (item->type) {
-		case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_DOODAD:
-			g_free (item);
+		case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_INVALID:
+		case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY:
 			break;
 
-		case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY:
-			key = (GkbdKeyboardDrawingKey *) item;
-			if (key->keycode < drawing->xkb->min_key_code ||
-			    key->keycode > drawing->xkb->max_key_code)
-				g_free (key);
-			/* otherwise it's part of the array */
+		case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY_EXTRA:
+		case GKBD_KEYBOARD_DRAWING_ITEM_TYPE_DOODAD:
+			g_free (item);
 			break;
 		}
 	}
@@ -1832,6 +1851,7 @@ xkb_state_notify_event_filter (GdkXEvent * gdkxev,
 
 	if (((XEvent *) gdkxev)->type == drawing->xkb_event_type) {
 		XkbEvent *kev = (XkbEvent *) gdkxev;
+		GtkAllocation allocation;
 		switch (kev->any.xkb_type) {
 		case XkbStateNotify:
 			if (((kev->state.changed & modifier_change_mask) &&
@@ -1845,9 +1865,12 @@ xkb_state_notify_event_filter (GdkXEvent * gdkxev,
 				    g_new0 (GkbdKeyboardDrawingKey,
 					    drawing->xkb->max_key_code +
 					    1);
+
+				gtk_widget_get_allocation (GTK_WIDGET
+							   (drawing),
+							   &allocation);
 				size_allocate (GTK_WIDGET (drawing),
-					       &(GTK_WIDGET (drawing)->
-						 allocation), drawing);
+					       &allocation, drawing);
 
 				init_keys_and_doodads (drawing);
 				init_colors (drawing);
@@ -1864,44 +1887,38 @@ xkb_state_notify_event_filter (GdkXEvent * gdkxev,
 
 				for (i = 0;
 				     i <=
-				     drawing->xkb->indicators->
-				     phys_indicators; i++)
-					if (drawing->
-					    physical_indicators[i] != NULL
+				     drawing->xkb->
+				     indicators->phys_indicators; i++)
+					if (drawing->physical_indicators[i]
+					    != NULL
 					    && (iev->changed & 1 << i)) {
 						gint state =
-						    (iev->
-						     state & 1 << i) !=
-						    FALSE;
+						    (iev->state & 1 << i)
+						    != FALSE;
 
 						if ((state
-						     && !drawing->
-						     physical_indicators
+						     &&
+						     !drawing->physical_indicators
 						     [i]->on) || (!state
 								  &&
-								  drawing->
-								  physical_indicators
-								  [i]->
-								  on)) {
-							drawing->
-							    physical_indicators
+								  drawing->physical_indicators
+								  [i]->on))
+						{
+							drawing->physical_indicators
 							    [i]->on =
 							    state;
 							create_cairo
 							    (drawing);
 							draw_doodad
-							    (drawing->
-							     renderContext,
+							    (drawing->renderContext,
 							     drawing,
-							     drawing->
-							     physical_indicators
+							     drawing->physical_indicators
 							     [i]);
 							destroy_cairo
 							    (drawing);
 							invalidate_indicator_doodad_region
 							    (drawing,
-							     drawing->
-							     physical_indicators
+							     drawing->physical_indicators
 							     [i]);
 						}
 					}
@@ -1963,7 +1980,7 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 
 	drawing->display =
 	    GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
-	printf ("dpy: %p\n", drawing->display);
+	printf ("dpy: %p\n", (void *) drawing->display);
 
 	if (!XkbQueryExtension
 	    (drawing->display, &opcode, &drawing->xkb_event_type, &error,
@@ -2038,7 +2055,7 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 	init_colors (drawing);
 
 	/* required to get key events */
-	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (drawing), GTK_CAN_FOCUS);
+	gtk_widget_set_can_focus (GTK_WIDGET (drawing), TRUE);
 
 	gtk_widget_set_events (GTK_WIDGET (drawing),
 			       GDK_EXPOSURE_MASK | GDK_KEY_PRESS_MASK |
@@ -2047,9 +2064,9 @@ gkbd_keyboard_drawing_init (GkbdKeyboardDrawing * drawing)
 	g_signal_connect (G_OBJECT (drawing), "expose-event",
 			  G_CALLBACK (expose_event), drawing);
 	g_signal_connect_after (G_OBJECT (drawing), "key-press-event",
-			  G_CALLBACK (key_event), drawing);
+				G_CALLBACK (key_event), drawing);
 	g_signal_connect_after (G_OBJECT (drawing), "key-release-event",
-			  G_CALLBACK (key_event), drawing);
+				G_CALLBACK (key_event), drawing);
 	g_signal_connect (G_OBJECT (drawing), "button-press-event",
 			  G_CALLBACK (button_press_event), drawing);
 	g_signal_connect (G_OBJECT (drawing), "focus-out-event",
@@ -2142,15 +2159,9 @@ gkbd_keyboard_drawing_get_pixbuf (GkbdKeyboardDrawing * drawing)
 	return gdk_pixbuf_get_from_drawable (NULL, drawing->pixmap, NULL,
 					     0, 0, 0, 0,
 					     xkb_to_pixmap_coord (context,
-								  drawing->
-								  xkb->
-								  geom->
-								  width_mm),
+								  drawing->xkb->geom->width_mm),
 					     xkb_to_pixmap_coord (context,
-								  drawing->
-								  xkb->
-								  geom->
-								  height_mm));
+								  drawing->xkb->geom->height_mm));
 }
 
 /**
@@ -2181,11 +2192,12 @@ gkbd_keyboard_drawing_render (GkbdKeyboardDrawing * drawing,
 		cr,
 		drawing->renderContext->angle,
 		layout,
-		pango_font_description_copy (GTK_WIDGET (drawing)->style->
-					     font_desc),
+		pango_font_description_copy (gtk_widget_get_style
+					     (GTK_WIDGET
+					      (drawing))->font_desc),
 		1, 1,
-		&GTK_WIDGET (drawing)->style->dark[GTK_WIDGET_STATE
-						   (GTK_WIDGET (drawing))]
+		&gtk_widget_get_style (GTK_WIDGET (drawing))->dark
+		    [gtk_widget_get_state (GTK_WIDGET (drawing))]
 	};
 
 	if (!context_setup_scaling (&context, drawing, width, height,
@@ -2204,6 +2216,8 @@ gboolean
 gkbd_keyboard_drawing_set_keyboard (GkbdKeyboardDrawing * drawing,
 				    XkbComponentNamesRec * names)
 {
+	GtkAllocation allocation;
+
 	free_cdik (drawing);
 	if (drawing->xkb)
 		XkbFreeKeyboard (drawing->xkb, 0, TRUE);	/* free_all = TRUE */
@@ -2240,8 +2254,8 @@ gkbd_keyboard_drawing_set_keyboard (GkbdKeyboardDrawing * drawing,
 	init_keys_and_doodads (drawing);
 	init_colors (drawing);
 
-	size_allocate (GTK_WIDGET (drawing),
-		       &(GTK_WIDGET (drawing)->allocation), drawing);
+	gtk_widget_get_allocation (GTK_WIDGET (drawing), &allocation);
+	size_allocate (GTK_WIDGET (drawing), &allocation, drawing);
 	gtk_widget_queue_draw (GTK_WIDGET (drawing));
 
 	return TRUE;
@@ -2357,7 +2371,13 @@ gkbd_keyboard_drawing_begin_print (GtkPrintOperation * operation,
 				   XkbLayoutPreviewPrintData * data)
 {
 	/* We always print single-page documents */
+	GtkPrintSettings *settings =
+	    gtk_print_operation_get_print_settings (operation);
 	gtk_print_operation_set_n_pages (operation, 1);
+	if (!gtk_print_settings_has_key
+	    (settings, GTK_PRINT_SETTINGS_ORIENTATION))
+		gtk_print_settings_set_orientation (settings,
+						    GTK_PAGE_ORIENTATION_LANDSCAPE);
 }
 
 static void
