@@ -42,7 +42,9 @@ typedef struct _GkbdKeyboardDrawingRenderContext
  GkbdKeyboardDrawingRenderContext;
 
 typedef enum {
+	GKBD_KEYBOARD_DRAWING_ITEM_TYPE_INVALID = 0,
 	GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY,
+	GKBD_KEYBOARD_DRAWING_ITEM_TYPE_KEY_EXTRA,
 	GKBD_KEYBOARD_DRAWING_ITEM_TYPE_DOODAD
 } GkbdKeyboardDrawingItemType;
 
@@ -55,7 +57,7 @@ typedef enum {
 	GKBD_KEYBOARD_DRAWING_POS_FIRST =
 	    GKBD_KEYBOARD_DRAWING_POS_TOPLEFT,
 	GKBD_KEYBOARD_DRAWING_POS_LAST =
-	    GKBD_KEYBOARD_DRAWING_POS_BOTTOMRIGHT,
+	    GKBD_KEYBOARD_DRAWING_POS_BOTTOMRIGHT
 } GkbdKeyboardDrawingGroupLevelPosition;
 
 /* units are in xkb form */
@@ -128,6 +130,7 @@ struct _GkbdKeyboardDrawing {
 
 	GkbdKeyboardDrawingRenderContext *renderContext;
 
+	/* Indexed by keycode */
 	GkbdKeyboardDrawingKey *keys;
 
 	/* list of stuff to draw in priority order */
